/**********************************************************************
* spredio.h
* 
* Routines for screen painting and message generation
**********************************************************************
                    This file is part of

         STK -- The sprite toolkit -- version 1.1

              Copyright (C) Jari Karjala 1991

The sprite toolkit (STK) is a FreeWare toolkit for creating high
resolution sprite graphics with PCompatible hardware. This toolkit 
is provided as is without any warranty or such thing. See the file
COPYING for further information.

**********************************************************************
**********************************************************************/

/**********************************************************************
* Messages returned from get_msg
**********************************************************************/
typedef enum _msg {
    MSG_NONE,
    MSG_CLEAR, MSG_COPY_TO_MASK, MSG_COPY_TO_SHAPE, 
    MSG_SAVE, MSG_LOAD, MSG_QUIT, MSG_EXIT, MSG_NEXT, MSG_PREV,
    MSG_INVERT, MSG_HFLIP, MSG_VFLIP, MSG_OVERLAY, MSG_VMIRROR, MSG_HMIRROR,
    MSG_ROTATE, MSG_ROTATE90, MSG_FATTER, MSG_THINNER,
    MSG_ACTIVATE_MASK, MSG_ACTIVATE_SHAPE,
    MSG_BTN1_CLICK, MSG_BTN2_CLICK, MSG_BTN3_CLICK
} MSG;

#define DOT_FOREGROUND 1
#define DOT_BACKGROUND 0
#define DOT_OVERLAY    0x100

/**********************************************************************
* Print the message, clear old message
**********************************************************************/
void message(char *s,...);

/**********************************************************************
* Draw a point into i:th map (shape/mask) of sprite.
**********************************************************************/
void draw_point(SPRED_DATA *sdp, int x, int y, int i);

/**********************************************************************
* Draw the ind:th map (shape/mask) of the given sprite (bigmap too, 
* if active)
**********************************************************************/
void draw_map(SPRED_DATA *sdp, int ind);

/**********************************************************************
* Draw the screen layout and sprite maps.
* Must be called before any other draw function.
**********************************************************************/
void draw_screen(SPRED_DATA *sdp);

/**********************************************************************
* Checks for user interaction.
* 
* Return: The event message, and the *x & *y contain the click point
*          in big map coordinates (ie 0..sdp->w-1 and 0..sdp->h-1).
**********************************************************************/
MSG get_msg(SPRED_DATA *sdp, int *xp, int *yp);
