/**********************************************************************
* mouse.h
* 
* The mouse interface through INT 33.
* NOTE: The pointer is not visible in the 2nd page of a Hercules card
**********************************************************************
                    This file is part of

         STK -- The sprite toolkit -- version 1.1

              Copyright (C) Jari Karjala 1991

The sprite toolkit (STK) is a FreeWare toolkit for creating high
resolution sprite graphics with PCompatible hardware. This toolkit 
is provided as is without any warranty or such thing. See the file
COPYING for further information.

**********************************************************************
**********************************************************************/

#if     !defined(__MOUSE_H_)
#define __MOUSE_H_

/**********************************************************************
* Codes for mouse button presses: 
**********************************************************************/
#define BUTTON_LEFT         1
#define BUTTON_RIGHT        2
#define BUTTON_MS_MIDDLE    (BUTTON_RIGHT | BUTTON_LEFT)
#define BUTTON_MIDDLE       4


typedef unsigned int MOUSE_POINTER[2][16];
/**********************************************************************
* The mouse cursor datatype.
* First mask and then shape bitmap (must be WORDs) 
**********************************************************************/

/**********************************************************************
* Predefined mouse cursor shapes:
**********************************************************************/
#ifndef NO_EXTERNS
extern MOUSE_POINTER mouse_pointer_arrow;    /* HotSpot 1,1   */
extern MOUSE_POINTER mouse_pointer_hourglass;/* HotSpot 7,7  */
extern MOUSE_POINTER mouse_pointer_cross;    /* HotSpot 7,7   */
#else
#undef NO_EXTERNS
#endif

/***** Function prototypes *****/

int     mouse_initialize(void);
/**********************************************************************
* Initialize the mouse driver, set the pointer shape to arrow,
* set pointer window to full screen and enable cursor.
* Return: the number of buttons or 0 if no mouse driver detected
**********************************************************************/

/**********************************************************************
* For the following functions, see the mouse documention
**********************************************************************/
int     mouse_driver_init(void);                                /**  0 **/
void    mouse_show_pointer(void);                               /**  1 **/
void    mouse_hide_pointer(void);                               /**  2 **/
void    mouse_get_pointer_xy(int *x, int *y);                   /**  3 **/
void    mouse_get_rel_pointer_xy(int *x, int *y);               /** 11 **/
int     mouse_get_buttons(void);                                /**  3 **/
int     mouse_get_presses(int buttons);                         /**  5 **/
int     mouse_get_releases(int buttons);                        /**  6 **/
void    mouse_set_pointer_xy(int x, int y);                     /**  4 **/
void    mouse_set_sensitivity(int x, int y);                    /** 15 **/
void    mouse_set_pointer_shape(int HotX, int HotY, MOUSE_POINTER shape);
void    mouse_set_pointer_window(int a, int b, int c, int d);    /**  7 **/
void    mouse_disable_pointer_window(int a, int b, int c, int d);/** 16 **/
void    mouse_set_light_pen(int a);                         /** 13 & 14 **/

#endif
