/**********************************************************************
* spr_low.h
*
* The hardware dependent parts of the sprite support. 
**********************************************************************
                    This file is part of

          STK -- The sprite toolkit -- version 1.1

              Copyright (C) Jari Karjala 1991

The sprite toolkit (STK) is a FreeWare toolkit for creating high
resolution sprite graphics with PCompatible hardware. This toolkit 
is provided as is without any warranty or such thing. See the file
COPYING for further information.

**********************************************************************
**********************************************************************/

/**********************************************************************
* ADDR
* Return the address of the given (x,y) coordinate pair in the
* given graphics page.
* 
* x,y     X,Y coordinates
* page    The graphics page (0/1)
*
* Return: far pointer into the screen.
**********************************************************************/

/**********************************************************************
* PUT
* Put the given shape into the given address using the given mask and
* saving the old contents into the given buffer. The shape and mask
* data must be combined as described in the file sprP.h and they
* must reside in the same segment as save buffer.
* 
* shape   The address of the combined shape/mask data
* dest    The destination address in the frame buffer
* save    The address of the save buffer
* w       The width of the shape in bytes
* h       The height of the shape in pixels
**********************************************************************/

/**********************************************************************
* ERASE
* Erase the sprite from screen by putting the saved data back
* 
* dest    The destination address in the frame buffer
* save    The address of the save buffer
* w       The width of the shape in bytes
* h       The height of the shape in pixels
**********************************************************************/


/*** Hercules graphics card **/
void spr_low_herc_put(WORD w, WORD h,
                      BYTE far *dest, BYTE far *shape, BYTE far *save);
void spr_low_herc_erase(WORD w, WORD h, BYTE far *dest, BYTE far *save);

/*** EGA/VGA graphics card in B/W 640x350 mode **/
void spr_low_ega_mono_put(WORD w, WORD h, 
                          BYTE far *dest, BYTE far *shape, BYTE far *save);
void spr_low_ega_mono_erase(WORD w, WORD h, BYTE far *dest, BYTE far *save);

